@@ -13,16 +13,7 @@ from logging.handlers import BufferingHandler
 from smtplib import SMTP, SMTPException
 import yaml
 
-
-JAVA_PARAMS = {
-    'memory': os.getenv('JAVA_MEM', '4096m'),
-    'fml_confirm': os.getenv('FML_CONFIRM', 'cancel'),
-    'log_level': os.getenv('MINECRAFT_LOG_LEVEL', 'warning'),
-    'log4j_xml': os.getenv(
-        'MINECRAFT_LOG4J_CONF',
-        os.path.join(os.path.dirname(os.path.realpath(__file__)), 'extra_files/log4j2.xml')
-    )
-}
+LOGGER = logging.getLogger('mine-init')
 
 
 class MakeFileHandler(logging.FileHandler):
@@ -30,7 +21,17 @@ class MakeFileHandler(logging.FileHandler):
     | A file handler class that ensures the logging dir is precreated
     """
     def __init__(self, filename, mode='a', encoding=None, delay=0):
-        os.makedirs(os.path.dirname(filename), exist_ok=True)
+        try:
+            os.makedirs(os.path.dirname(filename), exist_ok=True)
+        except PermissionError as error:
+            LOGGER.error(
+                'Could not log to %s, permission was denied. Check the permissions'
+                ' or specify a different log_dir.\n'
+                'Stack Trace: \n%s\n',
+                filename,
+                error
+            )
+            sys.exit(1)
         logging.FileHandler.__init__(self, filename, mode, encoding, delay)
 
 
@@ -204,32 +205,6 @@ class PropertyFileManager:
                               'or escalate your privileges as necessary.', self.filename)
 
 
-class MOTDGenerator:
-    """
-    | An MOTD Generator.
-    """
-    def __init__(self):
-        self.motd = False
-        if os.environ.get('MOTD'):
-            self.motd = str(os.environ.get('MOTD'))
-
-    def generate(self):
-        """
-        | Check for the MOTD property and return, else auto-generate and return.
-        | :return: `string` Message of the day.
-        """
-
-        if self.motd:
-            return self.motd
-
-        game_type = os.environ.get('PACK_NAME', os.environ.get('GAME_TYPE'.lower(), 'modded'))
-        version = os.environ.get('VERSION', 'Unknown Minecraft Version')
-        creator = os.environ.get('PACK_CREATOR', 'Routh.IO')
-        host = os.environ.get('GAME_HOST', 'Routh.IO')
-
-        return 'A %s %s Minecraft server by %s. Hosted by %s' % (game_type, version, creator, host)
-
-
 class PackFileUpdater:
     """
     | Updates files in from a pack distribution to a server volume.
@@ -244,8 +219,8 @@ class PackFileUpdater:
         | Sync the /dist/pack/server directory with the /server directory.
         | :return: `boolean` This always returns true, for now.
         """
+        logpipe = LogPipe()
         try:
-            logpipe = LogPipe()
             with subprocess.Popen(
                     'rm -rf '
                     'mods '
@@ -258,8 +233,7 @@ class PackFileUpdater:
                     encoding="utf-8",
                     stdout=logpipe.fileno(),
                     stderr=logpipe.fileno(),
-                    executable='/bin/bash'
-                ) as result:
+                    executable='/bin/bash') as result:
                 result.wait()
                 logpipe.close()
         except FileNotFoundError as not_found_error:
@@ -267,14 +241,16 @@ class PackFileUpdater:
                 'Error accessing %s directory',
                 not_found_error.filename
             )
+            logpipe.close()
+            sys.exit(1)
 
         self.logger.info(
             'Syncing %s to %s',
             os.path.join(self.source, 'server/'),
             os.path.join(self.dest)
         )
+        logpipe = LogPipe()
         try:
-            logpipe = LogPipe()
             with subprocess.Popen(
                     'rsync -av --ignore-existing %s %s'
                     % (
@@ -285,8 +261,7 @@ class PackFileUpdater:
                     encoding="utf-8",
                     stdout=logpipe.fileno(),
                     stderr=logpipe.fileno(),
-                    executable='/bin/bash'
-                ) as result:
+                    executable='/bin/bash') as result:
                 result.wait()
                 logpipe.close()
         except FileNotFoundError as not_found_error:
@@ -294,6 +269,8 @@ class PackFileUpdater:
                 'Error accessing %s directory',
                 not_found_error.filename
             )
+            logpipe.close()
+            sys.exit(1)
 
         if os.path.exists(os.path.join(self.source, 'server/config/')):
             self.logger.info(
@@ -301,8 +278,8 @@ class PackFileUpdater:
                 os.path.join(self.source, 'server/config/'),
                 os.path.join(self.dest, 'config/')
             )
+            logpipe = LogPipe()
             try:
-                logpipe = LogPipe()
                 with subprocess.Popen(
                         'rsync -av %s %s'
                         % (
@@ -313,8 +290,7 @@ class PackFileUpdater:
                         encoding="utf-8",
                         stdout=logpipe.fileno(),
                         stderr=logpipe.fileno(),
-                        executable='/bin/bash'
-                    ) as result:
+                        executable='/bin/bash') as result:
                     result.wait()
                 result.wait()
                 logpipe.close()
@@ -323,7 +299,8 @@ class PackFileUpdater:
                     'Error accessing %s directory',
                     not_found_error.filename
                 )
-
+                logpipe.close()
+                sys.exit(1)
         return True
 
 
@@ -342,22 +319,21 @@ class PackManager:
         |
         | :return: `boolean` True or False based on success.
         """
+        logpipe = LogPipe()
         try:
-            logpipe = LogPipe()
             with subprocess.Popen(
                     'packmaker build-server '
                     '--cache-dir %s --build-dir %s %s' % (
                         self.cache_dir,
                         self.build_dir[0],
-                        ' '.join([s + 'packmaker.lock' for s in self.build_dir])
+                        ' '.join([os.path.join(s, 'packmaker.lock') for s in self.build_dir])
                     ),
                     shell=True,
                     encoding="utf-8",
                     cwd=self.build_dir[0],
                     stdout=logpipe.fileno(),
                     stderr=logpipe.fileno(),
-                    executable='/bin/bash'
-                ) as result:
+                    executable='/bin/bash') as result:
                 result.wait()
                 logpipe.close()
         except ModuleNotFoundError as pm_error:
@@ -366,7 +342,21 @@ class PackManager:
                 'See stack trace: \n\n %s',
                 pm_error
             )
-            return False
+            logpipe.close()
+            sys.exit(1)
+        except FileNotFoundError as file_error:
+            self.logger.error(
+                'Packmaker could not find one of the files. Please check the cache and '
+                'build_dir settings:\n'
+                'cache: %s\n'
+                'build_dirs: %s\n'
+                'Stack Trace: \n%s\n',
+                self.cache_dir,
+                self.build_dir,
+                file_error
+            )
+            logpipe.close()
+            sys.exit(1)
         return True
 
 
@@ -374,13 +364,13 @@ class JavaRunner:
     """
     | Launches the Minecraft server
     """
-    def __init__(self, server_dir, pack_dir):
+    def __init__(self, server_dir, pack_dir, java_params):
         self.server_dir = server_dir
         self.pack_dir = pack_dir
         self.logger = logging.getLogger('mine-init')
         self.pack_data = self.get_pack_data()
         self.logpipe = LogPipe()
-        self.server = self.start_jvm()
+        self.server = self.start_jvm(java_params)
 
     def get_pack_data(self):
         """
@@ -391,11 +381,11 @@ class JavaRunner:
             with subprocess.Popen(
                     'packmaker info -f yaml',
                     stdout=subprocess.PIPE,
+                    stderr=subprocess.PIPE,
                     shell=True,
                     encoding="utf-8",
                     cwd=self.pack_dir,
-                    executable='/bin/bash'
-                ) as result:
+                    executable='/bin/bash') as result:
                 result.wait()
                 return yaml.load(result.stdout, Loader=yaml.FullLoader)
         except FileNotFoundError as not_found_error:
@@ -403,6 +393,8 @@ class JavaRunner:
                 'Error accessing %s',
                 not_found_error.filename
             )
+            self.logpipe.close()
+            sys.exit(1)
 
     def signal_handler(self, _signal, _frame):
         """
@@ -420,24 +412,21 @@ class JavaRunner:
             self.server.kill()
             self.server.communicate()
             self.logpipe.close()
+            sys.exit(1)
         sys.exit(0)
 
-    def start_jvm(self):
+    def start_jvm(self, java_params):
         """
         | Starts the minecraft server.
         | :return: Minecraft server process.
 
         """
-        log_opts = '-Dlog4j.configurationFile=' + JAVA_PARAMS.get('log4j_xml')
-
-        if not os.getenv('MINECRAFT_LOG_LEVEL'):
-            # Ensures that the log level is set for log4j to parse.
-            os.putenv('MINECRAFT_LOG_LEVEL', JAVA_PARAMS.get('log_level'))
+        log_opts = '-Dlog4j.configurationFile=' + str(java_params.get('log4j_xml'))
 
         jvm_command = [
             'java',
-            '-Xmx%s' % JAVA_PARAMS.get('memory'),
-            '-Xms%s' % JAVA_PARAMS.get('memory'),
+            '-Xmx%s' % java_params.get('memory'),
+            '-Xms%s' % java_params.get('memory'),
             '-XX:+UseG1GC',
             '-Dsun.rmi.dgc.server.gcInterval=2147483646',
             '-XX:+UnlockExperimentalVMOptions',
@@ -445,7 +434,7 @@ class JavaRunner:
             '-XX:G1ReservePercent=20',
             '-XX:MaxGCPauseMillis=50',
             '-XX:G1HeapRegionSize=32M',
-            '-Dfml.queryResult=%s' % JAVA_PARAMS.get('fml_confirm'),
+            '-Dfml.queryResult=%s' % java_params.get('fml_confirm'),
             log_opts,
             '-jar',
             self.pack_data.get('forge_jarfile'),
@@ -454,6 +443,13 @@ class JavaRunner:
 
         self.logger.info('Launching Minecraft JVM with command: %s', ' '.join(jvm_command))
 
+        current_env = os.environ.copy()
+
+        # Ensures that the log level is set for log4j to parse, and
+        # ensure it's been overriden with config file or cli params if both
+        # they and the environment variable have been set.
+        current_env['LOG4J_LOG_LEVEL'] = str(java_params.get('log_level')).upper()
+
         return subprocess.Popen(
             ' '.join(jvm_command),
             stdout=self.logpipe.fileno(),
@@ -462,12 +458,13 @@ class JavaRunner:
             shell=True,
             encoding="utf-8",
             cwd=self.server_dir,
-            executable='/bin/bash'
+            executable='/bin/bash',
+            env=current_env
         )
 
     def main(self):
         """
-        | document me please.
+        | After server has initialized, wait for it to exit or CTRL-C to be called.
         """
         signal.signal(signal.SIGINT, self.signal_handler)
         self.logger.info('Press Ctrl+C to exit')
